﻿
IF OBJECT_ID('CsvToTable') IS NOT NULL DROP FUNCTION CsvToTable
GO

CREATE FUNCTION CsvToTable
(
@CSVString 	NVARCHAR(MAX),
@Delimiter	NVARCHAR(10)
)
RETURNS @tbl TABLE ([Value_Text] NVARCHAR(1000))
as

begin
declare @i int ,
	@j int
	select 	@i = 1
	while @i <= len(@CSVString)
	begin
		select	@j = charindex(@Delimiter, @CSVString, @i)
		if @j = 0
		begin
			select	@j = len(@CSVString) + 1
		end
		insert	@tbl select substring(@CSVString, @i, @j - @i)
		select	@i = @j + len(@Delimiter)
	end
	return
end
GO


GO

IF OBJECT_ID('Session_GetUserName') IS NOT NULL DROP FUNCTION Session_GetUserName
GO

CREATE FUNCTION Session_GetUserName () RETURNS USERNAME
AS BEGIN

	DECLARE @Context_Info VARCHAR(MAX)
	SET @Context_Info = CAST(CONTEXT_INFO() AS VARCHAR(MAX))

	IF CHARINDEX(CHAR(0), @Context_Info) > 0
		SET @Context_Info = SUBSTRING(@Context_Info, 0, CHARINDEX(CHAR(0), @Context_Info))
	RETURN COALESCE(@Context_Info, 'UNKNOWN')

END
GO
GO
IF OBJECT_ID('[Connectors_Extended]') IS NOT NULL
	DROP VIEW [Connectors_Extended]
GO

CREATE VIEW [Connectors_Extended]
AS

	SELECT C.[Connector_Id],
		   C.[Connector_Name],
		   C.[Connector_Url],
		   C.[Feed_Id],
		   F.[Feed_Name]

	  FROM [Connectors] C
		   INNER JOIN [Feeds] F
					  ON C.[Feed_Id] = F.[Feed_Id]
GO
GO
IF OBJECT_ID('[EventNotifiers]') IS NOT NULL DROP VIEW [EventNotifiers]
GO

CREATE VIEW [EventNotifiers] AS

SELECT N.[Notifier_Id]
      ,N.[Notifier_Name]
      ,N.[Notifier_Configuration]
      ,N.[Owner_User_Name]

      ,EO.[EventOccurence_Id]
      ,EO.[Event_Code]
      ,EO.[User_Name]
      ,EO.[Occurence_Date]

  FROM [EventOccurences] EO
       INNER JOIN [NotifierEventCodes] NEC
               ON EO.[Event_Code] = NEC.[Event_Code]
       INNER JOIN [Notifiers] N
               ON N.[Notifier_Id] = NEC.[Notifier_Id]
 WHERE N.[Active_Indicator] = 'Y'
GO
IF OBJECT_ID('[FeedDownloadJobs_Extended]') IS NOT NULL
	DROP VIEW [FeedDownloadJobs_Extended]
GO

CREATE VIEW [FeedDownloadJobs_Extended]
AS
	SELECT FDJ.[Connector_Id],
			 C.[Connector_Name],
			 C.[Connector_Url],
			 F.[Feed_Id],
			 F.[Feed_Name],
		   FDJ.[JobStatus_Code],
		   FDJ.[Package_Count],
		   FDJ.[PackagesReceived_Count],
		   FDJ.[Start_Date],
		   FDJ.[StartedBy_User_Name],
		   FDJ.[Completed_Date]
	  
	  FROM [FeedDownloadJobs] FDJ
		   INNER JOIN [Connectors] C
				   ON FDJ.[Connector_Id] = C.[Connector_Id]
		   INNER JOIN [Feeds] F
				   ON C.[Feed_Id] = F.[Feed_Id]
GO

GO
IF OBJECT_ID('[Packages_Extended]') IS NOT NULL
	DROP VIEW [Packages_Extended]
GO

CREATE VIEW [Packages_Extended]
AS

	SELECT 
	
		 P.[Package_Id]
		,P.[Version_Text]
		,P.[FileName_Text]
		,P.[TargetFrameworks_Text]
		,P.[Published_Date]
		,P.[Package_Size]
		,P.[PackageHash_Text]
		,P.[Feed_Id]
		,[Version_Download_Count] = P.[Download_Count]
		,[Total_Download_Count] = (SELECT SUM(P2.[Download_Count]) FROM [Packages] P2 WHERE P2.[Package_Id] = P.[Package_Id])
		,P.[NuspecFile_Bytes]
		,P.[NuspecxFile_Bytes]
		,P.[Cached_Indicator]
		
		,F.[Feed_Name]
		,F.[Feed_Description]

	  FROM [Packages] P

INNER JOIN [Feeds] F
	    ON P.[Feed_Id] = F.[Feed_Id]
GO
GO
IF OBJECT_ID('[Privileges_Extended]') IS NOT NULL
	DROP VIEW [Privileges_Extended]
GO

CREATE VIEW [Privileges_Extended]
AS

	SELECT P.[Privilege_Id]
		  ,P.[Principal_Name]
		  ,P.[PrincipalType_Code]
		  ,P.[BuiltIn_Directory_Indicator]
		  ,P.[PrivilegeType_Code]

		  ,R.[Role_Id]
		  ,R.[Role_Name]

		  ,F.[Feed_Id]
		  ,F.[Feed_Name]

	  FROM [Privileges] P
		   INNER JOIN [Roles] R
				   ON P.[Role_Id] = R.[Role_Id]

		   LEFT JOIN [Feeds] F
				  ON P.[Feed_Id] = F.[Feed_Id]
GO
GO
IF OBJECT_ID('[RoleTasks_Extended]') IS NOT NULL
	DROP VIEW [RoleTasks_Extended]
GO

CREATE VIEW [RoleTasks_Extended]
AS

	SELECT RT.[Role_Id],
		   R.[Role_Name],
		   T.[Task_Id],
		   T.[Task_Name]

	  FROM [RoleTasks] RT
		   INNER JOIN [Tasks] T
				   ON RT.[Task_Id] = T.[Task_Id]
		   INNER JOIN [Roles] R
				   ON RT.[Role_Id] = R.[Role_Id]

GO

GO
IF OBJECT_ID('TR__ConnectorFilters__ValidateSequence') IS NOT NULL DROP TRIGGER TR__ConnectorFilters__ValidateSequence
GO

CREATE TRIGGER TR__ConnectorFilters__ValidateSequence
       ON [ConnectorFilters] FOR UPDATE, INSERT, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM [ConnectorFilters] CF 
				   INNER JOIN (SELECT [Connector_Id], [Filter_Sequence],
									  [Actual_Filter_Sequence] = 
										   ROW_NUMBER() OVER (PARTITION BY [Connector_Id]
										   ORDER BY [Connector_Id], [Filter_Sequence])
									  FROM [ConnectorFilters]) CFS
						   ON CF.[Connector_Id] = CFS.[Connector_Id]
						  AND CF.[Filter_Sequence] = CFS.[Filter_Sequence]
			 WHERE CF.[Filter_Sequence] <> CFS.[Actual_Filter_Sequence]
			   AND ( 
			         EXISTS ( SELECT * FROM inserted
			                   WHERE CF.[Connector_Id] = [Connector_Id] )
					 OR 
			         EXISTS ( SELECT * FROM deleted
			                   WHERE CF.[Connector_Id] = [Connector_Id] )
				   )
		) 
	BEGIN

		RAISERROR('An invalid Filter_Sequence was provided for the ConnectorFilters.', 16, 1)
		ROLLBACK
	
	END
		
END
GO
GO
IF OBJECT_ID('[TR__EventOccurenceDetails__ValidateDetailValue]') IS NOT NULL DROP TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue]
GO

CREATE TRIGGER [TR__EventOccurenceDetails__ValidateDetailValue] 
       ON [EventOccurenceDetails] FOR UPDATE, INSERT 
AS 
BEGIN
	IF EXISTS
	(
		SELECT *
		  FROM inserted /*[EventOccurenceDetails]*/ EOD
			   INNER JOIN [EventTypeDetails] ETD
					   ON ETD.[Event_Code] = EOD.[Event_Code]
					  AND ETD.[Detail_Name] = EOD.[Detail_Name]
		 WHERE ETD.[Detail_Type] <> sql_variant_property(EOD.[Detail_Value], 'BaseType')
	)
		BEGIN
			RAISERROR('Event detail value/type mismatch.', 16, 1)
			ROLLBACK
		END
END
GO


GO
IF OBJECT_ID('TR__Users__ValidateAnonymous') IS NOT NULL DROP TRIGGER TR__Users__ValidateAnonymous
GO

CREATE TRIGGER TR__Users__ValidateAnonymous
       ON [Users] FOR UPDATE, DELETE
AS 
BEGIN

	IF EXISTS(
			SELECT *
			  FROM deleted
			 WHERE [User_Name] = 'Anonymous'
			 )
	   AND NOT EXISTS(
			SELECT *
			  FROM inserted
			 WHERE [User_Name] = 'Anonymous'
			 )
	BEGIN

		RAISERROR('Cannot delete or alter the anonymous user.', 16, 1)
		ROLLBACK
	
	END
		
END
GO
GO
IF OBJECT_ID('[__StoredProcInfo]') IS NOT NULL
	DROP TABLE [__StoredProcInfo]
GO

CREATE TABLE [__StoredProcInfo]
(
    [StoredProc_Name] SYSNAME,
    CONSTRAINT [PK__StoredProcInfo]
        PRIMARY KEY ([StoredProc_Name]),
    [Internal_Indicator] YNINDICATOR,
    [ReturnType_Name] VARCHAR(100) NULL,
    CONSTRAINT [CK__StoredProcInfo__ReturnType_Name_Domain]
        CHECK ([ReturnType_Name] IS NULL OR [ReturnType_Name] IN ('void', 'DataRow', 'DataTable', 'DataSet')),
    [DataTableNames_Csv] VARCHAR(255) NULL,
    [OutputPropertyNames_Csv] VARCHAR(255) NULL,
    [Description_Text] VARCHAR(MAX) NULL,
	[Remarks_Text] VARCHAR(MAX) NULL
)
GO

IF OBJECT_ID('[__AddStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__AddStoredProcInfo]
GO

CREATE PROCEDURE [__AddStoredProcInfo]
(
	@StoredProc_Name SYSNAME,
	@Internal_Indicator YNINDICATOR,
	@ReturnType_Name VARCHAR(100),
	@DataTableNames_Csv VARCHAR(255),
	@OutputPropertyNames_Csv VARCHAR(100),
	@Description_Text VARCHAR(MAX)
)
AS
BEGIN
	
	IF EXISTS(SELECT * FROM [__StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name)
		DELETE [__StoredProcInfo] WHERE [StoredProc_Name]=@StoredProc_Name

    INSERT INTO [__StoredProcInfo]
    (
         [StoredProc_Name]
        ,[Internal_Indicator]
        ,[ReturnType_Name]
        ,[DataTableNames_Csv]
        ,[OutputPropertyNames_Csv]
        ,[Description_Text]
    )
    VALUES
    (
         @StoredProc_Name
	    ,@Internal_Indicator
	    ,@ReturnType_Name
	    ,@DataTableNames_Csv
	    ,@OutputPropertyNames_Csv
	    ,@Description_Text
    )

END
GO

IF OBJECT_ID('[__GetStoredProcInfo]') IS NOT NULL
	DROP PROCEDURE [__GetStoredProcInfo]
GO

CREATE PROCEDURE [__GetStoredProcInfo]
(
	@StoredProc_Name SYSNAME = NULL
)
AS
BEGIN
    
    SET NOCOUNT ON
    
	IF @StoredProc_Name IS NULL
		SELECT * FROM [__StoredProcInfo]
	ELSE
		SELECT * FROM [__StoredProcInfo] WHERE [StoredProc_Name] = @StoredProc_Name

END
GO

GRANT EXECUTE ON [__GetStoredProcInfo] TO [ProGetUser_Role]
GO

EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ '__GetStoredProcInfo',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ '__StoredProcInfo',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ '__AddStoredProcInfo',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'HandleError',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[HandleError]') IS NOT NULL DROP PROCEDURE [HandleError]
GO

CREATE PROCEDURE [HandleError] AS BEGIN
   -- Return if there is no error information to retrieve.
    IF ERROR_NUMBER() IS NULL
        RETURN;

    DECLARE 
        @ErrorMessage    NVARCHAR(4000),
        @ErrorNumber     INT,
        @ErrorSeverity   INT,
        @ErrorState      INT,
        @ErrorLine       INT,
        @ErrorProcedure  NVARCHAR(200);

    -- Assign variables to error-handling functions that 
    -- capture information for RAISERROR.
    SELECT 
        @ErrorNumber = ERROR_NUMBER(),
        @ErrorSeverity = ERROR_SEVERITY(),
        @ErrorState = ERROR_STATE(),
        @ErrorLine = ERROR_LINE(),
        @ErrorProcedure = REPLACE(ISNULL(ERROR_PROCEDURE(), '-'),'`','''');

    -- Building the message string that will contain original
    -- error information.
    SELECT @ErrorMessage = N'%d`%d`%d`%s`%d`'+ ERROR_MESSAGE();

    -- Raise an error: msg_str parameter of RAISERROR will contain
    -- the original error information.
    RAISERROR 
        (
        @ErrorMessage, 
        @ErrorSeverity, 
        42,               
        @ErrorNumber,    -- parameter: original error number.
        @ErrorSeverity,  -- parameter: original error severity.
        @ErrorState,     -- parameter: original error state.
        @ErrorProcedure, -- parameter: original error procedure name.
        @ErrorLine       -- parameter: original error line number.
        );
END
GO


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_RaiseEvent',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Events_RaiseEvent]') IS NOT NULL
	DROP PROCEDURE [Events_RaiseEvent]
GO

CREATE PROCEDURE [Events_RaiseEvent]
(
	@Event_Code VARCHAR(8),
	@detail1 SQL_VARIANT = NULL,
	@detail2 SQL_VARIANT = NULL,
	@detail3 SQL_VARIANT = NULL,
	@detail4 SQL_VARIANT = NULL,
	@detail5 SQL_VARIANT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @User_Name USERNAME
		SET @User_Name = dbo.Session_GetUserName()

	INSERT INTO [EventOccurences]
	 ([Event_Code],[User_Name],[Occurence_Date])
	 VALUES
	 (@Event_Code, @User_Name, GETUTCDATE())

	DECLARE @EventOccurence_Id INT
		SET @EventOccurence_Id = SCOPE_IDENTITY()

	INSERT INTO [EventOccurenceDetails]
	 ([EventOccurence_Id],[Event_Code],[Detail_Name],[Detail_Value])
	SELECT @EventOccurence_Id
		  ,[Event_Code]
		  ,[Detail_Name]
		  ,CASE [Detail_Sequence]
				WHEN 1 THEN @detail1
				WHEN 2 THEN @detail2
				WHEN 3 THEN @detail3
				WHEN 4 THEN @detail4
				WHEN 5 THEN @detail5
				ELSE NULL END
	  FROM [EventTypeDetails]
	 WHERE [Event_Code] = @Event_Code

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetConfiguration',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Configuration',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the a complete list of ProGet configuration settings'
GO

IF OBJECT_ID('[Configuration_GetConfiguration]') IS NOT NULL DROP PROCEDURE [Configuration_GetConfiguration]
GO

CREATE PROCEDURE [Configuration_GetConfiguration] AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Configuration]
	ORDER BY [Scope_Name], [Key_Name]

END
GO

GRANT EXECUTE ON [Configuration_GetConfiguration] TO [ProGetUser_Role]
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_GetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Value_Text',
    /* Description_Text        */ 'Gets the specified ProGet configuration value'
GO


IF OBJECT_ID('[Configuration_GetValue]') IS NOT NULL
	DROP PROCEDURE [Configuration_GetValue]
GO

CREATE PROCEDURE [Configuration_GetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text VARCHAR(MAX) = NULL OUT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT @Value_Text = [Value_Text]
	  FROM [Configuration]
	 WHERE [Scope_Name] = @Scope_Name
	   AND [Key_Name] = @Key_Name

END
GO

GRANT EXECUTE ON [Configuration_GetValue] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Configuration_SetValue',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets a ProGet configuration value'
GO


IF OBJECT_ID('[Configuration_SetValue]') IS NOT NULL
	DROP PROCEDURE [Configuration_SetValue]
GO

CREATE PROCEDURE [Configuration_SetValue]
(
	@Scope_Name VARCHAR(250),
	@Key_Name VARCHAR(250),
	@Value_Text NVARCHAR(MAX) = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [Configuration]
	 WHERE [Scope_Name] = @Scope_Name
	   AND [Key_Name] = @Key_Name

	IF (@Value_Text IS NOT NULL) 
		INSERT INTO [Configuration]
			([Scope_Name], [Key_Name], [Value_Text])
		VALUES
			(@Scope_Name, @Key_Name, @Value_Text)

	EXEC [Events_RaiseEvent] 'SYSCFG',@Key_Name


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Configuration_SetValue] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Connectors_AddFilter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Adds a filter to a connector'
GO

IF OBJECT_ID('[Connectors_AddFilter]') IS NOT NULL
	DROP PROCEDURE [Connectors_AddFilter]
GO


CREATE PROCEDURE [Connectors_AddFilter]
(
	@Connector_Id INT,
	@Allow_Indicator YNINDICATOR,
	@Rule_Text VARCHAR(250),
	
	@Filter_Sequence INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
		SET @Filter_Sequence = 
		      (SELECT COALESCE(MAX([Filter_Sequence]),0)+1
		         FROM [ConnectorFilters]
		        WHERE [Connector_Id] = @Connector_Id)
		
		/* New Connector; create */
		INSERT INTO [ConnectorFilters] ([Connector_Id], [Filter_Sequence],[Allow_Indicator],[Rule_Text])
		VALUES (@Connector_Id, @Filter_Sequence, @Allow_Indicator, @Rule_Text)
		EXEC [Events_RaiseEvent] 'CONNWC', @Connector_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Connectors_AddFilter] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Connectors_CreateOrUpdateConnector',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new Connector in the system, or updates an existing one'
GO

IF OBJECT_ID('[Connectors_CreateOrUpdateConnector]') IS NOT NULL
	DROP PROCEDURE [Connectors_CreateOrUpdateConnector]
GO


SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO



CREATE PROCEDURE [Connectors_CreateOrUpdateConnector]
(
	@Connector_Id INT = NULL OUT,
	@Connector_Name NVARCHAR(100),
	@Connector_Url VARCHAR(250),
	@ConnectorFilters_Xml XML,
	@Feed_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT * FROM [Connectors] WHERE [Connector_Id] = @Connector_Id)
	BEGIN

		/* Existing Connector; update */
		UPDATE [Connectors]
		   SET [Connector_Name] = @Connector_Name
			  ,[Connector_Url] = @Connector_Url
			  ,[Feed_Id] = @Feed_Id
		 WHERE [Connector_Id] = @Connector_Id
		EXEC [Events_RaiseEvent] 'CONMDC', @Connector_Id
	
	END ELSE BEGIN
	
		/* New Connector; create */
		INSERT INTO [Connectors]
			([Connector_Name],[Connector_Url],[Feed_Id])
		VALUES
			(@Connector_Name,@Connector_Url,@Feed_Id)
		SET @Connector_Id = SCOPE_IDENTITY()
		EXEC [Events_RaiseEvent] 'CONNWC', @Connector_Id

	END
	
	/* Prepare to rebuild filters */
	DELETE [ConnectorFilters] WHERE [Connector_Id] = @Connector_Id

	/* Add in the filters */
	INSERT INTO [ConnectorFilters]
		([Connector_Id],[Filter_Sequence],[Allow_Indicator],[Rule_Text])
	SELECT @Connector_Id
		  ,CFX.[value]('@Filter_Sequence', 'INT') [Filter_Sequence]
		  ,CFX.[value]('@Allow_Indicator', 'CHAR(1)') [Allow_Indicator]
		  ,CFX.[value]('@Rule_Text', 'VARCHAR(250)') [Rule_Text]
	  FROM @ConnectorFilters_Xml.[nodes]('/ConnectorFilters/ConnectorFilter') AS ConnectorFiltersXml(CFX)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Connectors_CreateOrUpdateConnector] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Connectors_DeleteConnector',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified connector'
GO

IF OBJECT_ID('[Connectors_DeleteConnector]') IS NOT NULL
	DROP PROCEDURE [Connectors_DeleteConnector]
GO


CREATE PROCEDURE [Connectors_DeleteConnector]
(
	@Connector_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	
	BEGIN TRANSACTION
	
		DECLARE @Connector_Name NVARCHAR(250)
		 SELECT @Connector_Name = [Connector_Name]
		   FROM [Connectors] WHERE [Connector_Id] = @Connector_Id
		IF @@ROWCOUNT = 0 BEGIN ROLLBACK RETURN END
		
		IF EXISTS(
			SELECT *
			  FROM [FeedDownloadJobs]
			 WHERE [Connector_Id] = @Connector_Id AND [JobStatus_Code] = 'I')
		BEGIN
			RAISERROR('Cannot delete the connector because it is currently in use by a Feed Download Job.', 16, 1)
		END
		
		DELETE [FeedDownloadJobs] WHERE [Connector_Id] = @Connector_Id
	
		DELETE [ConnectorFilters] WHERE [Connector_Id] = @Connector_Id
		
		DELETE [Connectors] WHERE [Connector_Id] = @Connector_Id
		
		EXEC [Events_RaiseEvent] 'CONDLC', @Connector_Name
	
	COMMIT
		
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Connectors_DeleteConnector] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Connectors_GetConnector',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Connectors_Extended,ConnectorFilters',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified Connector'
GO

IF OBJECT_ID('[Connectors_GetConnector]') IS NOT NULL DROP PROCEDURE [Connectors_GetConnector]
GO

CREATE PROCEDURE Connectors_GetConnector( @Connector_Id INT )
 AS BEGIN

	SET NOCOUNT ON

	SELECT * FROM [Connectors_Extended] WHERE [Connector_Id] = @Connector_Id

	SELECT * FROM [ConnectorFilters] WHERE [Connector_Id] = @Connector_Id

END
GO

GRANT EXECUTE ON [Connectors_GetConnector] TO [ProGetUser_Role]
GO
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Connectors_GetConnectors',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Connectors_Extended,ConnectorFilters',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the Connectors and ConnectorFilters in the system'
GO

IF OBJECT_ID('[Connectors_GetConnectors]') IS NOT NULL DROP PROCEDURE [Connectors_GetConnectors]
GO

CREATE PROCEDURE Connectors_GetConnectors
(
	@Feed_Id INT
)
 AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Connectors_Extended]
	 WHERE [Feed_Id] = COALESCE(@Feed_Id, [Feed_Id])
	
	SELECT *
	  FROM [ConnectorFilters] CF
	       INNER JOIN [Connectors] C
	               ON CF.[Connector_Id] = C.[Connector_Id]
	 WHERE C.[Feed_Id] = COALESCE(@Feed_Id, [Feed_Id])

END
GO

GRANT EXECUTE ON [Connectors_GetConnectors] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Connectors_ResequenceFilter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Changes the sequence of the specified connector filter'
GO

IF OBJECT_ID('[Connectors_ResequenceFilter]') IS NOT NULL
	DROP PROCEDURE [Connectors_ResequenceFilter]
GO


CREATE PROCEDURE [Connectors_ResequenceFilter]
(
	@Connector_Id INT,
	@Filter_Sequence INT,
	@New_Filter_Sequence INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	
	RAISERROR('Not Implemented', 16, 1)
		
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Connectors_ResequenceFilter] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurence',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'EventOccurences,EventOccurenceDetails',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Get a specific event occurence along with its event details'
GO

IF OBJECT_ID('[Events_GetEventOccurence]') IS NOT NULL
	DROP PROCEDURE [Events_GetEventOccurence]
GO

CREATE PROCEDURE [Events_GetEventOccurence]
(
	@EventOccurence_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [EventOccurences]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id
	IF @@ROWCOUNT=0 RAISERROR('Event #%i does not exist.', 16,1,@EventOccurence_Id)

	SELECT *
	  FROM [EventOccurenceDetails]
	 WHERE [EventOccurence_Id] = @EventOccurence_Id

END
GO

GRANT EXECUTE ON [Events_GetEventOccurence] TO [ProGetUser_Role]

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Events_GetEventOccurences',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'EventOccurences',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the event occurences in the system for a specific application, or all applications, limited to a specified count if provided'
GO

IF OBJECT_ID('[Events_GetEventOccurences]') IS NOT NULL
	DROP PROCEDURE [Events_GetEventOccurences]
GO

CREATE PROCEDURE [Events_GetEventOccurences]
(
	@Event_Count INT = NULL
)
AS
BEGIN

	SET NOCOUNT ON
	
	IF @Event_Count IS NOT NULL
		SET ROWCOUNT @Event_Count

	SELECT *
	  FROM [EventOccurences]
	 ORDER BY [Occurence_Date] DESC

END
GO

GRANT EXECUTE ON [Events_GetEventOccurences] TO [ProGetUser_Role]

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'FeedAdapters_CreateOrUpdateFeedAdapter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new Feed Adapter in the system, or updates an existing one'
GO

IF OBJECT_ID('[FeedAdapters_CreateOrUpdateFeedAdapter]') IS NOT NULL
	DROP PROCEDURE [FeedAdapters_CreateOrUpdateFeedAdapter]
GO

CREATE PROCEDURE [FeedAdapters_CreateOrUpdateFeedAdapter]
(
	@Feed_Id INT,
	@FeedAdapter_Sequence INT = NULL OUT,

	@FeedAdapter_Name NVARCHAR(50),
	@FeedAdapterType_Code VARCHAR(8),
	@FeedAdapter_Configuration VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT * FROM [FeedAdapters] WHERE [Feed_Id] = @Feed_Id AND [FeedAdapter_Sequence] = @FeedAdapter_Sequence)
	BEGIN

		/* Existing FeedAdapter; update */
		UPDATE [FeedAdapters]
		   SET [FeedAdapter_Name] = @FeedAdapter_Name
		      ,[FeedAdapterType_Code] = @FeedAdapterType_Code
			  ,[FeedAdapter_Configuration] = @FeedAdapter_Configuration
		 WHERE [Feed_Id] = @Feed_Id
		   AND [FeedAdapter_Sequence] = @FeedAdapter_Sequence
		EXEC [Events_RaiseEvent] 'FEDMDF', @Feed_Id
	
	END ELSE BEGIN

		SET @FeedAdapter_Sequence = 
			(SELECT COALESCE(MAX([FeedAdapter_Sequence]),0) + 1
			   FROM [FeedAdapters] WHERE [Feed_Id] = @Feed_Id)
	
		/* New Connector; create */
		INSERT INTO [FeedAdapters]
			([Feed_Id], [FeedAdapter_Sequence], [FeedAdapter_Name], [FeedAdapterType_Code], [FeedAdapter_Configuration])
		VALUES
			(@Feed_Id, @FeedAdapter_Sequence, @FeedAdapter_Name, @FeedAdapterType_Code, @FeedAdapter_Configuration)
		EXEC [Events_RaiseEvent] 'FEDMDF', @Feed_Id

	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [FeedAdapters_CreateOrUpdateFeedAdapter] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'FeedAdapters_DeleteFeedAdapter',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new Feed Adapter in the system, or updates an existing one'
GO

IF OBJECT_ID('[FeedAdapters_DeleteFeedAdapter]') IS NOT NULL
	DROP PROCEDURE [FeedAdapters_DeleteFeedAdapter]
GO

CREATE PROCEDURE [FeedAdapters_DeleteFeedAdapter]
(
	@Feed_Id INT,
	@FeedAdapter_Sequence INT
)
AS
BEGIN

	SET NOCOUNT ON

	DELETE [FeedAdapters] WHERE [Feed_Id] = @Feed_Id AND [FeedAdapter_Sequence] = @FeedAdapter_Sequence


END
GO

GRANT EXECUTE ON [FeedAdapters_DeleteFeedAdapter] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'FeedAdapters_GetFeedAdapters',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'FeedAdapters',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of adapters associated with the specified feed'
GO

IF OBJECT_ID('[FeedAdapters_GetFeedAdapters]') IS NOT NULL DROP PROCEDURE [FeedAdapters_GetFeedAdapters]
GO

CREATE PROCEDURE [FeedAdapters_GetFeedAdapters]
(
	@Feed_Id INT
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT * FROM [FeedAdapters] WHERE [Feed_Id] = @Feed_Id

END
GO

GRANT EXECUTE ON [FeedAdapters_GetFeedAdapters] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'FeedDownloadJobs_DeleteJob',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a previously scheduled feed download job'
GO

IF OBJECT_ID('[FeedDownloadJobs_DeleteJob]') IS NOT NULL
	DROP PROCEDURE [FeedDownloadJobs_DeleteJob]
GO


SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


CREATE PROCEDURE [FeedDownloadJobs_DeleteJob]
(
	@Connector_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DELETE [FeedDownloadJobs]
	 WHERE [Connector_Id] = @Connector_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [FeedDownloadJobs_DeleteJob] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'FeedDownloadJobs_GetJobs',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the scheduled feed download jobs'
GO

IF OBJECT_ID('[FeedDownloadJobs_GetJobs]') IS NOT NULL
	DROP PROCEDURE [FeedDownloadJobs_GetJobs]
GO


SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


CREATE PROCEDURE [FeedDownloadJobs_GetJobs]
(
	@JobStatus_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

	SELECT *
	  FROM [FeedDownloadJobs_Extended]
	 WHERE [JobStatus_Code] = COALESCE(@JobStatus_Code, [JobStatus_Code])

END
GO

GRANT EXECUTE ON [FeedDownloadJobs_GetJobs] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'FeedDownloadJobs_ScheduleJob',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Schedules a feed download job'
GO

IF OBJECT_ID('[FeedDownloadJobs_ScheduleJob]') IS NOT NULL
	DROP PROCEDURE [FeedDownloadJobs_ScheduleJob]
GO


SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


CREATE PROCEDURE [FeedDownloadJobs_ScheduleJob]
(
	@Connector_Id INT,
	@Package_Count INT,
	@Start_Date DATETIME = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	INSERT INTO [FeedDownloadJobs]
	(
		[Connector_Id],
		[Package_Count],
		[PackagesReceived_Count],
		[JobStatus_Code],
		[Start_Date],
		[StartedBy_User_Name]
	)
	VALUES
	(
		@Connector_Id,
		@Package_Count,
		0,
		'P',
		COALESCE(@Start_Date, GETUTCDATE()),
		dbo.Session_GetUserName()
	)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [FeedDownloadJobs_ScheduleJob] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'FeedDownloadJobs_UpdateJob',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Internal'
GO

IF OBJECT_ID('[FeedDownloadJobs_UpdateJob]') IS NOT NULL
	DROP PROCEDURE [FeedDownloadJobs_UpdateJob]
GO


SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO


CREATE PROCEDURE [FeedDownloadJobs_UpdateJob]
(
	@Connector_Id INT,
	@Package_Count INT = NULL,
	@PackagesReceived_Count INT = NULL,
	@JobStatus_Code CHAR(1) = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	UPDATE [FeedDownloadJobs]
	   SET [Package_Count] = COALESCE(@Package_Count, [Package_Count]),
		   [PackagesReceived_Count] = COALESCE(@PackagesReceived_Count, [PackagesReceived_Count]),
		   [JobStatus_Code] = COALESCE(@JobStatus_Code, [JobStatus_Code])
	 WHERE [Connector_Id] = @Connector_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [FeedDownloadJobs_UpdateJob] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Feeds_CreateOrUpdateFeed',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new Feed in the system, or updates an existing one'
GO

IF OBJECT_ID('[Feeds_CreateOrUpdateFeed]') IS NOT NULL
	DROP PROCEDURE [Feeds_CreateOrUpdateFeed]
GO

CREATE PROCEDURE [Feeds_CreateOrUpdateFeed]
(
	@Feed_Id INT = NULL OUT,
	@Feed_Name NVARCHAR(50),
	@Feed_Description NVARCHAR(MAX),
	@Active_Indicator YNINDICATOR,
	@Cache_Connectors_Indicator YNINDICATOR,
	@FeedPathOverride_Text NVARCHAR(260),
	@APIKey_Text NVARCHAR(100),
	@SymbolServerEnabled_Indicator YNINDICATOR,
	@PackageStore_Configuration VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT * FROM [Feeds] WHERE [Feed_Id] = @Feed_Id)
	BEGIN

		/* Existing Feed; update */
		UPDATE [Feeds]
		   SET [Feed_Name] = @Feed_Name
			  ,[Active_Indicator] = @Active_Indicator
			  ,[Cache_Connectors_Indicator] = @Cache_Connectors_Indicator
			  ,[Feed_Description] = @Feed_Description
			  ,[FeedPathOverride_Text] = @FeedPathOverride_Text
			  ,[APIKey_Text] = @APIKey_Text
			  ,[SymbolServerEnabled_Indicator] = @SymbolServerEnabled_Indicator
			  ,[PackageStore_Configuration] = @PackageStore_Configuration
		 WHERE [Feed_Id] = @Feed_Id
		EXEC [Events_RaiseEvent] 'FEDMDF', @Feed_Id
	
	END ELSE BEGIN
	
		/* New Connector; create */
		INSERT INTO [Feeds]
			([Feed_Name],[Active_Indicator],[Cache_Connectors_Indicator],[Feed_Description],[FeedPathOverride_Text],[APIKey_Text],[SymbolServerEnabled_Indicator],[PackageStore_Configuration])
		VALUES
			(@Feed_Name,@Active_Indicator,@Cache_Connectors_Indicator,@Feed_Description,@FeedPathOverride_Text,@APIKey_Text,@SymbolServerEnabled_Indicator,@PackageStore_Configuration)
		SET @Feed_Id = SCOPE_IDENTITY()
		EXEC [Events_RaiseEvent] 'FEDNWF', @Feed_Id

	END
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Feeds_CreateOrUpdateFeed] TO [ProGetUser_Role]
GO
	   


GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Feeds_DeleteFeed',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified feed'
GO

IF OBJECT_ID('[Feeds_DeleteFeed]') IS NOT NULL
	DROP PROCEDURE [Feeds_DeleteFeed]
GO


CREATE PROCEDURE [Feeds_DeleteFeed]
(
	@Feed_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	
	BEGIN TRANSACTION
	
		DECLARE @Feed_Name NVARCHAR(50)
		 SELECT @Feed_Name = [Feed_Name]
		   FROM [Feeds] WHERE [Feed_Id] = @Feed_Id
		IF @@ROWCOUNT = 0 BEGIN ROLLBACK RETURN END
		
		DELETE
		  FROM [FeedAdapters]
		 WHERE [Feed_Id] = @Feed_Id

		DELETE
		  FROM [ConnectorFilters]
		 WHERE [Connector_Id] IN ( SELECT [Connector_Id] FROM [Connectors] C WHERE C.[Feed_Id] = @Feed_Id )
	
		DELETE
		  FROM [FeedDownloadJobs]
		 WHERE [Connector_Id] IN ( SELECT [Connector_Id] FROM [Connectors] C WHERE C.[Feed_Id] = @Feed_Id )
	
		DELETE [Connectors]
		 WHERE [Feed_Id] = @Feed_Id
		 
		DELETE [Privileges]
		 WHERE [Feed_Id] = @Feed_Id
		 
		DELETE [Packages]
		 WHERE [Feed_Id] = @Feed_Id
		 
		DELETE [Feeds]
		 WHERE [Feed_Id] = @Feed_Id
		
		EXEC [Events_RaiseEvent] 'FEDDLF', @Feed_Name
	
	COMMIT
		
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Feeds_DeleteFeed] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Feeds_GetFeed',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Feeds,Connectors_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a feed and all of its associated connectors'
GO

IF OBJECT_ID('[Feeds_GetFeed]') IS NOT NULL DROP PROCEDURE [Feeds_GetFeed]
GO

CREATE PROCEDURE [Feeds_GetFeed]
(
	@Feed_Id INT
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT *
	  FROM [Feeds]
	 WHERE [Feed_Id] = @Feed_Id

	SELECT *
	  FROM [Connectors_Extended]
	 WHERE [Feed_Id] = @Feed_Id

END
GO

GRANT EXECUTE ON [Feeds_GetFeed] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Feeds_GetFeedByName',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Feeds,Connectors_Extended,FeedLicenseUrls',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of a feed and all of its associated connectors'
GO

IF OBJECT_ID('[Feeds_GetFeedByName]') IS NOT NULL DROP PROCEDURE [Feeds_GetFeedByName]
GO

CREATE PROCEDURE [Feeds_GetFeedByName]
(
	@Feed_Name NVARCHAR(50)
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT *
	  FROM [Feeds]
	 WHERE [Feed_Name] = @Feed_Name

	SELECT *
	  FROM [Connectors_Extended]
	 WHERE [Feed_Name] = @Feed_Name
	 
	SELECT FLU.*
	  FROM [FeedLicenseUrls] FLU
	 INNER JOIN [Feeds] F
	         ON F.[Feed_Id] = FLU.[Feed_Id]
	 WHERE F.[Feed_Name] = @Feed_Name

END
GO

GRANT EXECUTE ON [Feeds_GetFeedByName] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Feeds_GetFeeds',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Feeds',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the feeds'
GO

IF OBJECT_ID('[Feeds_GetFeeds]') IS NOT NULL DROP PROCEDURE [Feeds_GetFeeds]
GO

CREATE PROCEDURE [Feeds_GetFeeds]
(
	@IncludeInactive_Indicator YNINDICATOR
)
AS
BEGIN
	
	SET NOCOUNT ON

	IF @IncludeInactive_Indicator = 'Y'
	BEGIN

		SELECT *
		  FROM [Feeds]
		 ORDER BY [Feed_Name]

	END ELSE BEGIN

		SELECT *
		  FROM [Feeds]
		 WHERE [Active_Indicator] = 'Y'
		 ORDER BY [Feed_Name]
	
	END

END
GO

GRANT EXECUTE ON [Feeds_GetFeeds] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Feeds_GetLicenseUrls',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'FeedLicenseUrls',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the license urls for a feed'
GO

IF OBJECT_ID('[Feeds_GetLicenseUrls]') IS NOT NULL DROP PROCEDURE [Feeds_GetLicenseUrls]
GO

CREATE PROCEDURE [Feeds_GetLicenseUrls]
(
	@Feed_Id INT
)
AS
BEGIN
	
	SET NOCOUNT ON

    SELECT * FROM [FeedLicenseUrls] WHERE [Feed_Id] = @Feed_Id

END
GO

GRANT EXECUTE ON [Feeds_GetLicenseUrls] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Feeds_SetLicenseUrls',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Sets the license urls for a particlar feed.'
GO

IF OBJECT_ID('[Feeds_SetLicenseUrls]') IS NOT NULL
	DROP PROCEDURE [Feeds_SetLicenseUrls]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO



CREATE PROCEDURE [Feeds_SetLicenseUrls]
(
	@Feed_Id INT,
    @LicenseUrls_Xml XML
)
AS
BEGIN
     
    SET NOCOUNT ON
    BEGIN TRY
	    BEGIN TRANSACTION

        DELETE [FeedLicenseUrls] WHERE [Feed_Id] = @Feed_Id

	    INSERT INTO [FeedLicenseUrls]
		    ([Feed_Id],[LicenseUrl_Text])
	    SELECT @Feed_Id
		      ,LUX.[value]('.', 'VARCHAR(250)') [LicenseUrl_Text]
	      FROM @LicenseUrls_Xml.[nodes]('/LicenseUrls/LicenseUrl') AS LicenseUrlsXml(LUX)
        
        EXEC [Events_RaiseEvent] 'FEDMDF', @Feed_Id

	    COMMIT
    END TRY BEGIN CATCH
	    IF XACT_STATE()<>0 ROLLBACK
	    EXEC [HandleError]
    END CATCH

END
GO

GRANT EXECUTE ON [Feeds_SetLicenseUrls] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_CreateOrUpdateNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new notifier, or updates an existing one'
GO

IF OBJECT_ID('[Notifiers_CreateOrUpdateNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_CreateOrUpdateNotifier]
GO

CREATE PROCEDURE [Notifiers_CreateOrUpdateNotifier]
(
	@Notifier_Id INT = NULL,
	@Notifier_Name NVARCHAR(50),
	@Notifier_Description NVARCHAR(MAX),
	@Notifier_Configuration NVARCHAR(MAX),
	@Owner_User_Name USERNAME,
	@Active_Indicator YNINDICATOR,
	@EventCodeList_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Notifier_Id IS NULL BEGIN
		INSERT INTO [Notifiers]
				   ([Notifier_Name]
				   ,[Notifier_Description]
				   ,[Notifier_Configuration]
				   ,[Owner_User_Name]
				   ,[Active_Indicator])
			 VALUES
				   (@Notifier_Name
				   ,@Notifier_Description
				   ,@Notifier_Configuration
				   ,@Owner_User_Name
				   ,@Active_Indicator)
		SET @Notifier_Id = SCOPE_IDENTITY()
	END ELSE BEGIN
		UPDATE [Notifiers] SET
		   [Notifier_Name]=@Notifier_Name
		   ,[Notifier_Description] = @Notifier_Description
		   ,[Notifier_Configuration] = @Notifier_Configuration
		   ,[Active_Indicator] = @Active_Indicator
		WHERE [Notifier_Id] = @Notifier_Id
		IF @@ROWCOUNT=0 RAISERROR('Notifier #%i does not exist.',16,1,@Notifier_Id)

		DELETE [NotifierEventCodes] 
		 WHERE [Notifier_Id] = @Notifier_Id
	END

	INSERT INTO [NotifierEventCodes] ([Notifier_Id],[Event_Code])
	SELECT @Notifier_Id, V.[Value_Text]
	  FROM dbo.CsvToTable(@EventCodeList_Csv,',') V

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_CreateOrUpdateNotifier] TO [ProGetUser_Role]
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_DeleteNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a notifier from the system'
GO

IF OBJECT_ID('[Notifiers_DeleteNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_DeleteNotifier]
GO

CREATE PROCEDURE [Notifiers_DeleteNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [NotifierEventCodes] WHERE [Notifier_Id] = @Notifier_Id

	DELETE [Notifiers] WHERE [Notifier_Id] = @Notifier_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Notifiers_DeleteNotifier] TO [ProGetUser_Role]
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifier',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Notifiers,NotifierEventCodes',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a notifier and a list of event codes that the notifier is subscribed to'
GO

IF OBJECT_ID('[Notifiers_GetNotifier]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifier]
GO

CREATE PROCEDURE [Notifiers_GetNotifier]
(
	@Notifier_Id INT
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Notifier_Id] = @Notifier_Id
	IF @@ROWCOUNT=0 RAISERROR('Notifier %i does not exist.', 16,1,@Notifier_Id)

	SELECT * FROM [NotifierEventCodes]
	 WHERE [Notifier_Id] = @Notifier_Id

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifier] TO [ProGetUser_Role]
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Notifiers_GetNotifiers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Notifiers',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of notifiers for the specified user, or all users if no user is specified'
GO

IF OBJECT_ID('[Notifiers_GetNotifiers]') IS NOT NULL
	DROP PROCEDURE [Notifiers_GetNotifiers]
GO

CREATE PROCEDURE [Notifiers_GetNotifiers]
(
	@Owner_User_Name USERNAME = NULL
)
AS
BEGIN

SET NOCOUNT ON

	SELECT * FROM [Notifiers]
	 WHERE [Owner_User_Name] = COALESCE(@Owner_User_Name,[Owner_User_Name])
	   OR ([Owner_User_Name] IS NULL AND @Owner_User_Name IS NULL)

END
GO

GRANT EXECUTE ON [Notifiers_GetNotifiers] TO [ProGetUser_Role]

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Packages_CreateOrUpdatePackage',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new package, or updates an existing one'
GO

IF OBJECT_ID('[Packages_CreateOrUpdatePackage]') IS NOT NULL
	DROP PROCEDURE [Packages_CreateOrUpdatePackage]
GO

CREATE PROCEDURE [Packages_CreateOrUpdatePackage]
(
	@Package_Id NVARCHAR(255),
	@Version_Text NVARCHAR(50),
	@FileName_Text NVARCHAR(MAX),
	@TargetFrameworks_Text NVARCHAR(255),
	@Published_Date DATETIME,
	@Package_Size BIGINT,
	@PackageHash_Text VARCHAR(255),
	@Feed_Id INT,
	@Cached_Indicator YNINDICATOR,
	@NuspecFile_Bytes VARBINARY(MAX),
	@NuspecxFile_Bytes VARBINARY(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(
		SELECT * 
		  FROM [Packages] 
		 WHERE [Package_Id] = @Package_Id 
		   AND [Version_Text] = @Version_Text
		)
	BEGIN

		UPDATE [Packages]
		   SET [Feed_Id] = @Feed_Id
			  ,[FileName_Text] = @FileName_Text
			  ,[TargetFrameworks_Text] = @TargetFrameworks_Text
			  ,[Published_Date] = @Published_Date
			  ,[Package_Size] = @Package_Size
			  ,[PackageHash_Text] = @PackageHash_Text
			  ,[Cached_Indicator] = @Cached_Indicator
		      ,[NuspecFile_Bytes] = @NuspecFile_Bytes
		      ,[NuspecxFile_Bytes] = @NuspecxFile_Bytes
		 WHERE [Package_Id] = @Package_Id 
		   AND [Version_Text] = @Version_Text

	END

	ELSE
	BEGIN

		INSERT INTO [Packages] 
		(
			[Package_Id]
		   ,[Version_Text]
		   ,[FileName_Text]
		   ,[TargetFrameworks_Text]
		   ,[Published_Date]
		   ,[Package_Size]
		   ,[PackageHash_Text]
		   ,[Feed_Id]
		   ,[Download_Count]
		   ,[Cached_Indicator]
		   ,[NuspecFile_Bytes]
		   ,[NuspecxFile_Bytes]
		)
		VALUES
		(
			@Package_Id
		   ,@Version_Text
		   ,@FileName_Text
		   ,@TargetFrameworks_Text
		   ,@Published_Date
	       ,@Package_Size
		   ,@PackageHash_Text
		   ,@Feed_Id
		   ,0
		   ,@Cached_Indicator
		   ,@NuspecFile_Bytes
		   ,@NuspecxFile_Bytes
		)
	END


	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Packages_CreateOrUpdatePackage] TO [ProGetUser_Role]
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Packages_DeletePackage',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ 'Packages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a specified package'
GO

IF OBJECT_ID('[Packages_DeletePackage]') IS NOT NULL DROP PROCEDURE [Packages_DeletePackage]
GO

CREATE PROCEDURE [Packages_DeletePackage]
(
	@FileName_Text NVARCHAR(MAX)
)
AS
BEGIN
	
	SET NOCOUNT ON

	DELETE [Packages]
	 WHERE [FileName_Text] = @FileName_Text

END
GO

GRANT EXECUTE ON [Packages_DeletePackage] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Packages_GetPackages',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Packages_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all of the packages, optionally filtered by feed'
GO

IF OBJECT_ID('[Packages_GetPackages]') IS NOT NULL DROP PROCEDURE [Packages_GetPackages]
GO

CREATE PROCEDURE [Packages_GetPackages]
(
	@Feed_Id INT
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT * 
	  FROM [Packages_Extended]
	 WHERE [Feed_Id] = COALESCE(@Feed_Id, [Feed_Id])

END
GO

GRANT EXECUTE ON [Packages_GetPackages] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Packages_IncrementDownloadCount',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ '',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Increments the download count of a local or cached remote package'
GO

IF OBJECT_ID('[Packages_IncrementDownloadCount]') IS NOT NULL DROP PROCEDURE [Packages_IncrementDownloadCount]
GO

CREATE PROCEDURE [Packages_IncrementDownloadCount]
(
	@Package_Id NVARCHAR(255),
	@Version_Text NVARCHAR(50)
)
AS
BEGIN
	
	SET NOCOUNT ON

	UPDATE [Packages]
	   SET [Download_Count] = [Download_Count] + 1
	 WHERE [Package_Id] = @Package_Id
	   AND [Version_Text] = @Version_Text

END
GO

GRANT EXECUTE ON [Packages_IncrementDownloadCount] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_AddPrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Privilege_Id',
    /* Description_Text        */ 'Grants or denies the specified principal the ability to perform a certain role'
GO

IF OBJECT_ID('[Security_AddPrivilege]') IS NOT NULL
	DROP PROCEDURE [Security_AddPrivilege]
GO

CREATE PROCEDURE [Security_AddPrivilege]
(
	@Privilege_Id INT = NULL OUT,
	@BuiltIn_Directory_Indicator YNINDICATOR,
	@Principal_Name USERNAME,
	@PrincipalType_Code CHAR(1),
	@Role_Id INT,
	@Feed_Id INT,
	@PrivilegeType_Code CHAR(1)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Catch duplicate data */
	IF EXISTS(SELECT *
				FROM [Privileges]
			   WHERE [Principal_Name] = @Principal_Name
				 AND [BuiltIn_Directory_Indicator] = @BuiltIn_Directory_Indicator
				 AND [PrincipalType_Code] = @PrincipalType_Code
				 AND [Role_Id] = @Role_Id
				 AND (	[Feed_Id] = @Feed_Id
					 OR COALESCE([Feed_Id], @Feed_Id) IS NULL )
				 AND [PrivilegeType_Code] = @PrivilegeType_Code )
	BEGIN
	    ROLLBACK
		RETURN
    END

	INSERT INTO [Privileges]
		([BuiltIn_Directory_Indicator]
		,[Principal_Name]
		,[PrincipalType_Code]
		,[Role_Id]
		,[Feed_Id]
		,[PrivilegeType_Code])
	VALUES
		(@BuiltIn_Directory_Indicator
		,@Principal_Name
		,@PrincipalType_Code
		,@Role_Id
		,@Feed_Id
		,@PrivilegeType_Code)
	SET @Privilege_Id = SCOPE_IDENTITY()

	EXEC [Events_RaiseEvent] 'SECNWP', @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_AddPrivilege] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_CreateOrUpdateRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Role_Id',
    /* Description_Text        */ 'Creates a new role defined by the specified tasks'
GO

IF OBJECT_ID('[Security_CreateOrUpdateRole]') IS NOT NULL
	DROP PROCEDURE [Security_CreateOrUpdateRole]
GO

CREATE PROCEDURE [Security_CreateOrUpdateRole]
(
	@Role_Id INT = NULL OUT,
	@Role_Name NVARCHAR(50),
	@Tasks_Csv VARCHAR(MAX)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Role_Id IS NULL
	BEGIN
		/* Add new role */
		INSERT INTO [Roles]
			([Role_Name])
		VALUES
			(@Role_Name)
		SET @Role_Id = SCOPE_IDENTITY()

		EXEC [Events_RaiseEvent] 'SECNWR', @Role_Id

	END
	ELSE
	BEGIN
		/* Update existing role */
		UPDATE [Roles]
		   SET [Role_Name] = @Role_Name
		 WHERE [Role_Id] = @Role_Id

		/* Remove existing tasks for the role */
		DELETE [RoleTasks]
		 WHERE [Role_Id] = @Role_Id

		EXEC [Events_RaiseEvent] 'SECMDR', @Role_Id
	END

	/* Add the appropriate role tasks */
	INSERT INTO [RoleTasks]
		([Role_Id]
		,[Task_Id])
	SELECT @Role_Id,[Value_Text]
	  FROM [CsvToTable](@Tasks_Csv, ',')

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Security_CreateOrUpdateRole] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_DeleteRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes the specified role from the system'
GO

IF OBJECT_ID('[Security_DeleteRole]') IS NOT NULL
	DROP PROCEDURE [Security_DeleteRole]
GO

CREATE PROCEDURE [Security_DeleteRole]
(
	@Role_Id INT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Role_Name SQL_VARIANT
	    SET @Role_Name = (SELECT [Role_Name] FROM [Roles] WHERE [Role_Id] = @Role_Id)

	DELETE [Privileges]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [RoleTasks]
	 WHERE [Role_Id] = @Role_Id
	
	DELETE [Roles]
	 WHERE [Role_Id] = @Role_Id

	IF @@ROWCOUNT>0 EXEC [Events_RaiseEvent] 'SECDLR',  @Role_Name
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Security_DeleteRole] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetPrivileges',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Privileges_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of privileges assigned in the system'
GO

IF OBJECT_ID('[Security_GetPrivileges]') IS NOT NULL
	DROP PROCEDURE [Security_GetPrivileges]
GO

CREATE PROCEDURE [Security_GetPrivileges]
(
	@BuiltIn_Directory_Indicator YNINDICATOR,
	@PrincipalType_Code CHAR(1) = NULL
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Privileges_Extended]
	 WHERE [PrincipalType_Code] = COALESCE(@PrincipalType_Code, [PrincipalType_Code])
	   AND [BuiltIn_Directory_Indicator] = @BuiltIn_Directory_Indicator
END
GO

GRANT EXECUTE ON [Security_GetPrivileges] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the specified role in the database, along with all the tasks that comprise the role'
GO

IF OBJECT_ID('[Security_GetRole]') IS NOT NULL
	DROP PROCEDURE [Security_GetRole]
GO

CREATE PROCEDURE [Security_GetRole]
(
	@Role_Id INT
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	 WHERE [Role_Id] = @Role_Id
	IF @@ROWCOUNT=0 RAISERROR('Role #%i does not exist.',16,1,@Role_Id)


	SELECT *
	  FROM [RoleTasks_Extended]
	 WHERE [Role_Id] = @Role_Id

END
GO

GRANT EXECUTE ON [Security_GetRole] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetRoles',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Roles,RoleTasks_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the roles in the system, along with all the tasks comprising each role'
GO

IF OBJECT_ID('[Security_GetRoles]') IS NOT NULL
	DROP PROCEDURE [Security_GetRoles]
GO

CREATE PROCEDURE [Security_GetRoles]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Roles]
	ORDER BY [Role_Name]

	SELECT *
	  FROM [RoleTasks_Extended]
	ORDER BY [Task_Name]

END
GO

GRANT EXECUTE ON [Security_GetRoles] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_GetUserPrivileges',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Privileges_Extended',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the list of privileges specifically assigned to a user or any specified groups'
GO

IF OBJECT_ID('[Security_GetUserPrivileges]') IS NOT NULL
	DROP PROCEDURE [Security_GetUserPrivileges]
GO

CREATE PROCEDURE [Security_GetUserPrivileges]
(
	@BuiltIn_Directory_Indicator YNINDICATOR,
	@User_Name USERNAME,
	@GroupNames_Csv NVARCHAR(MAX)
)
AS
BEGIN

	SET NOCOUNT ON

	/* Privileges */
	SELECT *
	  FROM [Privileges_Extended]
	 WHERE [BuiltIn_Directory_Indicator] = @BuiltIn_Directory_Indicator
	   AND (   ([Principal_Name] = @User_Name AND [PrincipalType_Code] = 'U') 
		    OR ([Principal_Name] IN (SELECT [Value_Text] FROM [CsvToTable](@GroupNames_Csv, ',') )
			    AND [PrincipalType_Code] = 'G' ) )

END
GO

GRANT EXECUTE ON [Security_GetUserPrivileges] TO [ProGetUser_Role]
GO

GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForAnyTask',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Authorized_Indicator',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform a task'
GO


IF OBJECT_ID('Security_IsAuthorizedForAnyTask') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForAnyTask]
GO
CREATE PROCEDURE [Security_IsAuthorizedForAnyTask] (
	@BuiltIn_Directory_Indicator YNINDICATOR,
	@User_Name USERNAME,
	@GroupNames_Csv NVARCHAR(MAX),
	@Authorized_Indicator YNINDICATOR OUT
) AS
	SET NOCOUNT ON

	IF EXISTS(
		SELECT *
		  FROM [Privileges] P
			   INNER JOIN [RoleTasks] RT
					   ON RT.[Role_Id] = P.[Role_Id]
			   INNER JOIN [Tasks] T
					   ON T.[Task_Id] = RT.[Task_Id]
		 WHERE (( /* User */
				   P.[PrincipalType_Code] = 'U'
				   AND P.[Principal_Name] = @User_Name
				 ) OR (
				   /* Group */ 
				   P.[PrincipalType_Code] = 'G'
				   AND P.[Principal_Name] IN 
					 (SELECT [Value_Text]
						FROM [CsvToTable](@GroupNames_Csv, ',') ) 
				))
			AND P.[BuiltIn_Directory_Indicator] = @BuiltIn_Directory_Indicator
			AND P.[PrivilegeType_Code] <> 'D'
	) SET @Authorized_Indicator = 'Y'
	  ELSE SET @Authorized_Indicator = 'N'

GO

GRANT EXECUTE ON [Security_IsAuthorizedForAnyTask] TO [ProGetUser_Role]
GO
GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_IsAuthorizedForTask',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Authorized_Indicator',
    /* Description_Text        */ 'Determines whether or not the specified user with specified groups has authorized to perform a task'
GO


IF OBJECT_ID('Security_IsAuthorizedForTask') IS NOT NULL DROP PROCEDURE [Security_IsAuthorizedForTask]
GO
CREATE PROCEDURE Security_IsAuthorizedForTask (
	@BuiltIn_Directory_Indicator YNINDICATOR,
	@User_Name USERNAME,
	@GroupNames_Csv NVARCHAR(MAX),
	@Task_Id INT,
	@Feed_Id INT,
	@Authorized_Indicator YNINDICATOR OUT
) AS
	SET NOCOUNT ON

	SET @Authorized_Indicator = COALESCE(
	(
		SELECT TOP 1 CASE [PrivilegeType_Code] 
                     WHEN 'G' THEN 'Y' WHEN 'D' THEN 'N' END
		  FROM [Privileges] P
			   INNER JOIN [RoleTasks] RT
					   ON RT.[Role_Id] = P.[Role_Id]
			   INNER JOIN [Tasks] T
					   ON T.[Task_Id] = RT.[Task_Id]
		 WHERE (( /* User */
				   P.[PrincipalType_Code] = 'U'
				   AND P.[Principal_Name] = @User_Name
				 ) OR (
				   /* Group */ 
				   P.[PrincipalType_Code] = 'G'
				   AND P.[Principal_Name] IN 
					 (SELECT [Value_Text]
						FROM [CsvToTable](@GroupNames_Csv, ',') ) 
				))
			AND ((
			      /* If no app group specified, only system-scope is acceptable */			
			      (@Feed_Id IS NULL AND P.[Feed_Id] IS NULL)
			      OR
			      /* If app group is specified, system or app group scopes are acceptable */
			      (@Feed_Id IS NOT NULL AND
			         (P.[Feed_Id] IS NULL
			          OR P.[Feed_Id] = @Feed_Id)
			      )
				))
			AND T.[Task_Id] = @Task_Id
			AND P.[BuiltIn_Directory_Indicator] = @BuiltIn_Directory_Indicator
		 ORDER BY CHARINDEX([PrincipalType_Code], 'UG'), /* Users (Explicit) always override Groups (Implicit) */
                  CASE WHEN P.[Feed_Id] IS NOT NULL THEN 0 ELSE 1 END, /* Scopes always override system */
                  CHARINDEX([PrivilegeType_Code], 'DG') /* Deny Always overrides Grant */
	),'N')

GO

GRANT EXECUTE ON [Security_IsAuthorizedForTask] TO [ProGetUser_Role]
GO
GO
EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Security_RemovePrivilege',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a privilege from the system'
GO

IF OBJECT_ID('[Security_RemovePrivilege]') IS NOT NULL
	DROP PROCEDURE [Security_RemovePrivilege]
GO

CREATE PROCEDURE [Security_RemovePrivilege]
(
	@Privilege_Id INT
)
AS
BEGIN
SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	DELETE [Privileges]
	 WHERE [Privilege_Id] = @Privilege_Id

	IF @@ROWCOUNT>0 EXEC [Events_RaiseEvent] 'SECDLP', @Privilege_Id

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [Security_RemovePrivilege] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user group in the system'
GO

IF OBJECT_ID('[Users_CreateGroup]') IS NOT NULL
	DROP PROCEDURE [Users_CreateGroup]
GO

CREATE PROCEDURE [Users_CreateGroup]
(
	@Group_Name NVARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Users]
			   WHERE [User_Name] = @Group_Name)
		RAISERROR('Cannot create group "%s"; a user of the same name exists', 16, 1)

	INSERT INTO [Groups]
		([Group_Name])
	VALUES
		(@Group_Name)

	EXEC [Events_RaiseEvent] 'USRNWG',  @Group_Name
	
	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_CreateGroup] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user in the system, or updates an existing one'
GO

IF OBJECT_ID('[Users_CreateOrUpdateUser]') IS NOT NULL
	DROP PROCEDURE [Users_CreateOrUpdateUser]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [Users_CreateOrUpdateUser]
(
	@User_Name USERNAME,
	@Display_Name NVARCHAR(100),
	@Email_Address VARCHAR(250) = NULL,
	@Groups_Xml XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Groups]
			   WHERE [Group_Name] = @User_Name)
		RAISERROR('Cannot create user "%s"; a group of the same name exists', 16, 1)

	IF EXISTS(SELECT *
				FROM [Users]
			   WHERE [User_Name] = @User_Name)
	BEGIN

		/* Existing user; update */
		UPDATE [Users]
		   SET [Display_Name] = @Display_Name
			  ,[Email_Address] = @Email_Address
		 WHERE [User_Name] = @User_Name
		EXEC [Events_RaiseEvent] 'USRMDU', @User_Name

	END
	ELSE
	BEGIN

		/* New user; create */
		INSERT INTO [Users]
			([User_Name]
			,[Display_Name]
			,[Email_Address])
		VALUES
			(@User_Name
			,@Display_Name
			,@Email_Address)
		EXEC [Events_RaiseEvent] 'USRNWU', @User_Name

	END

	/* Prepare to rebuild group memberships */
	DELETE [UserGroups] WHERE [User_Name] = @User_Name

	/* Add in the group memberships */
	INSERT INTO [UserGroups] ([User_Name] ,[Group_Name])
	SELECT @User_Name
		  ,GX.[value]('.', 'VARCHAR(50)') [Group_Name]
	  FROM @Groups_Xml.[nodes]('/groups/group') AS GroupsXml(GX)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_CreateOrUpdateUser] TO [ProGetUser_Role]
GO
GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user group from the system, but does not delete the users who belong to the group'
GO

IF OBJECT_ID('[Users_DeleteGroup]') IS NOT NULL
	DROP PROCEDURE [Users_DeleteGroup]
GO

CREATE PROCEDURE [Users_DeleteGroup]
(
	@Group_Name NVARCHAR(50)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	/* Remove group privileges */
	DELETE [Privileges]
	 WHERE [Principal_Name] = @Group_Name
	   AND [PrincipalType_Code] = 'G'
	   AND [BuiltIn_Directory_Indicator] = 'Y'

	/* Remove group users */
	DELETE [UserGroups]
	 WHERE [Group_Name] = @Group_Name

	/* Remove group */
	DELETE [Groups]
	 WHERE [Group_Name] = @Group_Name

	EXEC [Events_RaiseEvent] 'USRDLG',  @Group_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_DeleteGroup] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_DeleteUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Deletes a user from the system'
GO

IF OBJECT_ID('[Users_DeleteUser]') IS NOT NULL
	DROP PROCEDURE [Users_DeleteUser]
GO

CREATE PROCEDURE [Users_DeleteUser]
(
	@User_Name USERNAME
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION


	DELETE [Privileges]
	 WHERE [Principal_Name] = @User_Name
	   AND [PrincipalType_Code] = 'U'
	   AND [BuiltIn_Directory_Indicator] = 'Y'

	DELETE [UserGroups]
	 WHERE [User_Name] = @User_Name

	DELETE [Users]
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRDLU', @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_DeleteUser] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetGroups',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Groups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of user groups in the system'
GO

IF OBJECT_ID('[Users_GetGroups]') IS NOT NULL
	DROP PROCEDURE [Users_GetGroups]
GO

CREATE PROCEDURE [Users_GetGroups]
AS
BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Groups]
	ORDER BY [Group_Name]

END
GO

GRANT EXECUTE ON [Users_GetGroups] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the details of the specified user, and a list of all the groups the user belongs to'
GO

IF OBJECT_ID('[Users_GetUser]') IS NOT NULL DROP PROCEDURE [Users_GetUser]
GO

CREATE PROCEDURE [Users_GetUser]
(
	@User_Name USERNAME
)
AS
BEGIN
	
	SET NOCOUNT ON

	SELECT *
	  FROM [Users]
	 WHERE [User_Name] = @User_Name

	SELECT *
	  FROM [UserGroups]
	 WHERE [User_Name] = @User_Name

END
GO

GRANT EXECUTE ON [Users_GetUser] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsers',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataSet',
    /* DataTableNames_Csv      */ 'Users,UserGroups',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets all the users in the system, along with all the groups in the system'
GO

IF OBJECT_ID('[Users_GetUsers]') IS NOT NULL DROP PROCEDURE [Users_GetUsers]
GO

CREATE PROCEDURE Users_GetUsers
 AS BEGIN

	SET NOCOUNT ON

	SELECT *
	  FROM [Users]

	SELECT *
	  FROM [UserGroups]

END
GO

GRANT EXECUTE ON [Users_GetUsers] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_GetUsersInGroup',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Users',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets a list of users in the specified group'
GO

IF OBJECT_ID('[Users_GetUsersInGroup]') IS NOT NULL
	DROP PROCEDURE [Users_GetUsersInGroup]
GO

CREATE PROCEDURE [Users_GetUsersInGroup]
(
    @Group_Name VARCHAR(50)
)
AS
BEGIN

	SET NOCOUNT ON

	SELECT U.*
	  FROM [UserGroups] UG
	INNER JOIN [Users] U ON UG.[User_Name] = U.[User_Name]
	WHERE [Group_Name] = @Group_Name
	ORDER BY [User_Name]

END
GO

GRANT EXECUTE ON [Users_GetUsersInGroup] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_SetPassword',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ NULL,
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Stored procedure for internal use only.'
GO

IF OBJECT_ID('[Users_SetPassword]') IS NOT NULL
	DROP PROCEDURE [Users_SetPassword]
GO

CREATE PROCEDURE [Users_SetPassword]
(
	@User_Name USERNAME,
	@Password_Text VARCHAR(20)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION
	
	DECLARE @Salt_Bytes BINARY(10)
	DECLARE @Password_Bytes BINARY(20)
	
	SET @Salt_Bytes = CAST(NEWID() AS BINARY(10))
	SET @Password_Bytes = HASHBYTES('SHA1', @Salt_Bytes + CAST(@Password_Text AS VARBINARY))
	
	UPDATE [Users]
	   SET [Password_Bytes] = @Password_Bytes
	      ,[Salt_Bytes] = @Salt_Bytes
	 WHERE [User_Name] = @User_Name

	EXEC [Events_RaiseEvent] 'USRMDU', @User_Name

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_SetPassword] TO [ProGetUser_Role]
GO

GO
EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_ValidateLogin',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Validated_Indicator',
    /* Description_Text        */ 'Validates the password of a specified user'
GO

IF OBJECT_ID('Users_ValidateLogin') IS NOT NULL DROP PROCEDURE Users_ValidateLogin
GO

CREATE PROCEDURE Users_ValidateLogin
(
	@User_Name USERNAME,
	@Password_Text VARCHAR(20),
	@Validated_Indicator YNINDICATOR = NULL OUT
) AS BEGIN

	SET NOCOUNT ON

	/* Set defaults */
	SET @Validated_Indicator = 'N'

	IF EXISTS(SELECT * FROM [Users]
	           WHERE [User_Name] = @User_Name
	             AND HASHBYTES('SHA1', [Salt_Bytes] + CAST(@Password_Text AS VARBINARY)) = [Password_Bytes])
		SET @Validated_Indicator = 'Y'


END
GO

GRANT EXECUTE ON Users_ValidateLogin TO ProGetUser_Role
GO
